@@ -2,7 +2,7 @@ package
     util;
 use strict;
 use warnings;
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 require XSLoader;
 XSLoader::load('util', $VERSION);
 1;
@@ -440,6 +440,119 @@ Returns true if all elements match the comparison. Returns true for empty arrays
 
 Returns true if no element matches the comparison.
 
+=head1 CALLBACK REGISTRY
+
+The callback registry provides named callbacks that can be used with
+the C<*_cb> functions. This avoids Perl callback overhead for common
+predicates and enables XS modules to register C-level callbacks for
+maximum performance.
+
+=head2 Built-in Predicates
+
+All built-in predicates are prefixed with C<:> to distinguish them
+from user-registered callbacks:
+
+    :is_defined     - SvOK check
+    :is_true        - SvTRUE check  
+    :is_false       - !SvTRUE check
+    :is_ref         - SvROK check
+    :is_array       - Array reference
+    :is_hash        - Hash reference
+    :is_code        - Code reference
+    :is_positive    - Numeric > 0
+    :is_negative    - Numeric < 0
+    :is_zero        - Numeric == 0
+    :is_even        - Integer divisible by 2
+    :is_odd         - Integer not divisible by 2
+    :is_empty       - Undefined, empty string, empty array, or empty hash
+    :is_nonempty    - Defined and non-empty
+    :is_string      - Defined, not a reference
+    :is_number      - Looks like a number
+    :is_integer     - Integer value
+
+=head2 any_cb, all_cb, none_cb
+
+    my $bool = any_cb(\@numbers, ':is_positive');
+    my $bool = all_cb(\@numbers, ':is_even');
+    my $bool = none_cb(\@numbers, ':is_negative');
+
+Like C<any>, C<all>, and C<none> but use a registered callback by name.
+No Perl callback overhead - runs entirely in C.
+
+=head2 first_cb
+
+    my $found = first_cb(\@numbers, ':is_positive');
+
+Returns the first element for which the callback returns true.
+Returns undef if no element matches.
+
+=head2 grep_cb
+
+    my @positives = grep_cb(\@numbers, ':is_positive');
+
+Returns all elements for which the callback returns true.
+
+=head2 count_cb
+
+    my $n = count_cb(\@numbers, ':is_positive');
+
+Counts elements for which the callback returns true.
+
+=head2 partition_cb
+
+    my ($pass, $fail) = partition_cb(\@numbers, ':is_positive');
+
+Splits an array into two arrayrefs: the first contains elements
+matching the predicate, the second contains non-matching elements.
+Returns two arrayrefs.
+
+=head2 final_cb
+
+    my $last = final_cb(\@numbers, ':is_positive');
+
+Returns the last element for which the callback returns true.
+Searches from the end of the array for efficiency. Returns undef
+if no element matches.
+
+=head2 register_callback
+
+    register_callback('divisible_by_3', sub { $_[0] % 3 == 0 });
+
+Registers a Perl coderef as a named callback. The coderef receives
+the element as its first argument. Names cannot start with C<:>
+(reserved for built-ins) and cannot re-register existing names.
+
+=head2 has_callback
+
+    if (has_callback('divisible_by_3')) { ... }
+
+Returns true if a callback with the given name is registered.
+
+=head2 list_callbacks
+
+    my $callbacks = list_callbacks();
+
+Returns an arrayref of all registered callback names.
+
+=head2 XS Callback Registration
+
+External XS modules can register C-level callbacks for maximum performance.
+Include the header in your XS code:
+
+    #include "util_callbacks.h"
+
+Then register callbacks:
+
+    static bool my_is_valid(pTHX_ SV *elem) {
+        return SvOK(elem) && SvIV(elem) > 0;
+    }
+
+    BOOT:
+        util_register_predicate_xs("my_is_valid", my_is_valid);
+
+The C callback avoids all Perl overhead - no call_sv, no stack manipulation.
+See C<xs/util/util_callbacks.h> for the full API.
+
 =head1 DATA MANIPULATION
 
 These functions transform and extract data from arrays and hashes.
