@@ -136,6 +136,20 @@ static NumericHeap* get_numeric_heap(pTHX_ SV *obj) {
     return NULL;
 }
 
+/* Try to get numeric heap, return NULL if not a numeric heap */
+static NumericHeap* try_get_numeric_heap(pTHX_ SV *obj) {
+    MAGIC *mg;
+    if (!SvROK(obj)) return NULL;
+    mg = mg_find(SvRV(obj), PERL_MAGIC_ext);
+    while (mg) {
+        if (mg->mg_virtual == &numeric_heap_vtbl) {
+            return (NumericHeap*)mg->mg_ptr;
+        }
+        mg = mg->mg_moremagic;
+    }
+    return NULL;
+}
+
 /* ============================================
    PART 1: RAW ARRAY API (fastest)
    Operates directly on Perl arrays
@@ -906,11 +920,28 @@ static int heap_free(pTHX_ SV *sv, MAGIC *mg) {
     return 0;
 }
 
-/* Custom OP implementations */
+/* Custom OP implementations - handle both regular and numeric heaps */
 static OP* pp_heap_push(pTHX) {
     dSP;
     SV *val_sv = POPs;
     SV *heap_sv = TOPs;
+
+    /* Check for numeric heap first */
+    NumericHeap *nh = try_get_numeric_heap(aTHX_ heap_sv);
+    if (nh) {
+        NV val = SvNV(val_sv);
+        nv_ensure_capacity(nh, nh->size + 1);
+        nh->data[nh->size] = val;
+        nh->size++;
+        if (nh->type == HEAP_MIN) {
+            nv_sift_up_min(nh, nh->size - 1);
+        } else {
+            nv_sift_up_max(nh, nh->size - 1);
+        }
+        RETURN;
+    }
+
+    /* Regular heap */
     Heap *h = get_heap(aTHX_ heap_sv);
     SV *value = newSVsv(val_sv);
 
@@ -925,6 +956,29 @@ static OP* pp_heap_push(pTHX) {
 static OP* pp_heap_pop(pTHX) {
     dSP;
     SV *heap_sv = TOPs;
+
+    /* Check for numeric heap first */
+    NumericHeap *nh = try_get_numeric_heap(aTHX_ heap_sv);
+    if (nh) {
+        if (nh->size == 0) {
+            SETs(&PL_sv_undef);
+            RETURN;
+        }
+        NV result = nh->data[0];
+        nh->size--;
+        if (nh->size > 0) {
+            nh->data[0] = nh->data[nh->size];
+            if (nh->type == HEAP_MIN) {
+                nv_sift_down_min(nh, 0);
+            } else {
+                nv_sift_down_max(nh, 0);
+            }
+        }
+        SETs(sv_2mortal(newSVnv(result)));
+        RETURN;
+    }
+
+    /* Regular heap */
     Heap *h = get_heap(aTHX_ heap_sv);
 
     if (h->size == 0) {
@@ -948,6 +1002,19 @@ static OP* pp_heap_pop(pTHX) {
 static OP* pp_heap_peek(pTHX) {
     dSP;
     SV *heap_sv = TOPs;
+
+    /* Check for numeric heap first */
+    NumericHeap *nh = try_get_numeric_heap(aTHX_ heap_sv);
+    if (nh) {
+        if (nh->size == 0) {
+            SETs(&PL_sv_undef);
+        } else {
+            SETs(sv_2mortal(newSVnv(nh->data[0])));
+        }
+        RETURN;
+    }
+
+    /* Regular heap */
     Heap *h = get_heap(aTHX_ heap_sv);
 
     if (h->size == 0) {
@@ -961,6 +1028,15 @@ static OP* pp_heap_peek(pTHX) {
 static OP* pp_heap_size(pTHX) {
     dSP;
     SV *heap_sv = TOPs;
+
+    /* Check for numeric heap first */
+    NumericHeap *nh = try_get_numeric_heap(aTHX_ heap_sv);
+    if (nh) {
+        SETs(sv_2mortal(newSViv(nh->size)));
+        RETURN;
+    }
+
+    /* Regular heap */
     Heap *h = get_heap(aTHX_ heap_sv);
 
     SETs(sv_2mortal(newSViv(h->size)));
@@ -972,6 +1048,25 @@ static OP* pp_heap_func_push(pTHX) {
     dSP;
     SV *val_sv = TOPs;
     SV *heap_sv = TOPm1s;
+
+    /* Check for numeric heap first */
+    NumericHeap *nh = try_get_numeric_heap(aTHX_ heap_sv);
+    if (nh) {
+        NV val = SvNV(val_sv);
+        nv_ensure_capacity(nh, nh->size + 1);
+        nh->data[nh->size] = val;
+        nh->size++;
+        if (nh->type == HEAP_MIN) {
+            nv_sift_up_min(nh, nh->size - 1);
+        } else {
+            nv_sift_up_max(nh, nh->size - 1);
+        }
+        SP--;
+        SETs(heap_sv);
+        RETURN;
+    }
+
+    /* Regular heap */
     Heap *h = get_heap(aTHX_ heap_sv);
     SV *value = newSVsv(val_sv);
 
@@ -988,6 +1083,29 @@ static OP* pp_heap_func_push(pTHX) {
 static OP* pp_heap_func_pop(pTHX) {
     dSP;
     SV *heap_sv = TOPs;
+
+    /* Check for numeric heap first */
+    NumericHeap *nh = try_get_numeric_heap(aTHX_ heap_sv);
+    if (nh) {
+        if (nh->size == 0) {
+            SETs(&PL_sv_undef);
+            RETURN;
+        }
+        NV result = nh->data[0];
+        nh->size--;
+        if (nh->size > 0) {
+            nh->data[0] = nh->data[nh->size];
+            if (nh->type == HEAP_MIN) {
+                nv_sift_down_min(nh, 0);
+            } else {
+                nv_sift_down_max(nh, 0);
+            }
+        }
+        SETs(sv_2mortal(newSVnv(result)));
+        RETURN;
+    }
+
+    /* Regular heap */
     Heap *h = get_heap(aTHX_ heap_sv);
 
     if (h->size == 0) {
@@ -1011,8 +1129,16 @@ static OP* pp_heap_func_pop(pTHX) {
 static OP* pp_heap_func_peek(pTHX) {
     dSP;
     SV *heap_sv = TOPs;
-    Heap *h = get_heap(aTHX_ heap_sv);
 
+    /* Check for numeric heap first */
+    NumericHeap *nh = try_get_numeric_heap(aTHX_ heap_sv);
+    if (nh) {
+        SETs(nh->size > 0 ? sv_2mortal(newSVnv(nh->data[0])) : &PL_sv_undef);
+        RETURN;
+    }
+
+    /* Regular heap */
+    Heap *h = get_heap(aTHX_ heap_sv);
     SETs(h->size > 0 ? h->data[0] : &PL_sv_undef);
     RETURN;
 }
@@ -1020,8 +1146,16 @@ static OP* pp_heap_func_peek(pTHX) {
 static OP* pp_heap_func_size(pTHX) {
     dSP;
     SV *heap_sv = TOPs;
-    Heap *h = get_heap(aTHX_ heap_sv);
 
+    /* Check for numeric heap first */
+    NumericHeap *nh = try_get_numeric_heap(aTHX_ heap_sv);
+    if (nh) {
+        SETs(sv_2mortal(newSViv(nh->size)));
+        RETURN;
+    }
+
+    /* Regular heap */
+    Heap *h = get_heap(aTHX_ heap_sv);
     SETs(sv_2mortal(newSViv(h->size)));
     RETURN;
 }
