@@ -528,8 +528,7 @@ static OP* pp_object_new(pTHX) {
     obj_sv = newRV_noinc((SV*)obj_av);
     sv_bless(obj_sv, meta->stash);
 
-    /* Add object magic for flags */
-    add_object_magic(aTHX_ obj_sv);
+    /* Magic for lock/freeze is added lazily when first needed */
 
     SP = MARK;
     XPUSHs(obj_sv);
@@ -859,7 +858,7 @@ static XS(xs_object_new_fallback) {
 
     obj_sv = newRV_noinc((SV*)obj_av);
     sv_bless(obj_sv, meta->stash);
-    add_object_magic(aTHX_ obj_sv);
+    /* Magic for lock/freeze is added lazily when first needed */
 
     ST(0) = obj_sv;
     XSRETURN(1);
